<?php
namespace AngelicaO;
use MNC\MNC_API;

class Devices {
    public function __construct() {
    }


    /**
     * Gets all locations with their children locations, recursively. Sorted alphabetically
     * @return array
     * @throws \Exception
     */
    public static function buildDeviceData() {

        $sql =
            'SELECT devices.*, video_wall_layouts.name as `connectedLayout`, ' .
            '(SELECT COUNT(*) FROM video_wall_currently_playing_connections WHERE videoWallFullConfigId = devices.videoWallFullConfigId) as `connectedLayoutChanges` ' .
            ' FROM devices  ' .
            'LEFT JOIN video_wall_layouts ON (devices.selected_layout = video_wall_layouts.layout_id) ' .
            'WHERE mac NOT IN (SELECT mac FROM video_wall_config_devices)';
        /**
         * For now we'll live with the newly-complicated SQL, but if new products are released that require their
         * own SQL, consider using the meta_options table, or consider separate SQL statements and merging the results,
         * for readability's sake
         */
        $devices = DB::query($sql, array());

        //get the path of each device
        $locations = Locations::getAllLocations();

        for ($i = 0; $i < sizeof($devices); $i++) {

            //build the location data
            $device_path = '';

            if (isset($devices[$i]['location_id'])) {
                $device_location = Locations::utilFindLocationByID($devices[$i]['location_id'], $locations);
                if ($device_location !== null) {
                    $device_path = $device_location['name'];

                    $location_path = Locations::utilGetLocationPath($device_location, $locations);
                    if (sizeof($location_path)) {
                        $device_path .= ' > ' . implode(' > ', array_reverse($location_path));
                    }
                }
            }

            $devices[$i]['path'] = $device_path;
            //end build location data

            //TODO remove this when it's fixed
            //fix the malformatted model names that are hardcoded in muxlab products
            $devices[$i]['modelName'] = self::fixMalformattedModelName($devices[$i]['modelName']);


        }

        return $devices;
    }

    public static function getDevices() {
        $detected_devices = MNC_API::getDevices();

        $result_devices = self::processDevicePayloadFromMNC($detected_devices);
        
        //filter out the devices that the user doesn't have access to
        $filtered_devices = array();
        $blocked_devices = self::getBlockedUsersForDevices();

        $user_account = MNC_API::getAccount();
        foreach ($result_devices as $result_device) {
            $isBlocked = false;
            foreach ($blocked_devices as $blocked_device) {
                if ($blocked_device['mac'] == $result_device['mac'] && intval($blocked_device['user_id']) === intval($user_account['user_id'])) {
                    $isBlocked = true;
                }
            }

            if (empty($isBlocked)) {
                $filtered_devices[] = $result_device;
            }

        }
        
        return $filtered_devices;
    }

    /**
     * Gets all locations with their children locations, recursively. Sorted alpha.ly
     * @return array
     * @throws \Exception
     */
    private static function getSimpleDevices() {
        //doesn't check user permissions as this is an internal function that needs to know all devices
        $sql = 'SELECT * FROM devices WHERE isVideoWall = 0 AND mac NOT IN (SELECT mac FROM video_wall_config_devices) ';
        $devices = DB::query($sql, array());

        return $devices;
    }

    /**
     * Accepts an array describing the data {simpleDevices: [], videoWalls: []} from the MNC and updates the database accordingly. The videoWalls provided must already be nested according, with full configs on level 1, and sub-configs inside them. See front end code and comments
     * @param $MNCDevices
     * @return array formatted array of devices after processing. Is meant to be used by replacing the $scope.devices object in JS Controller
     * @throws \Exception
     */
    private static function processDevicePayloadFromMNC($MNCDevices) {

        $original_devices = self::buildDeviceData();//the devices before processing the new data

        if (empty($MNCDevices['simpleDevices'])) {
            $MNCDevices['simpleDevices'] = array();
        }
        self::processSimpleDevicePayloadFromMNC($MNCDevices['simpleDevices']);


        if (empty($MNCDevices['videoWalls'])) {
            $MNCDevices['videoWalls'] = array();
        }
        VideoWall::processVideoWallPayloadFromMNC($MNCDevices['videoWalls']);

        $result_devices = self::buildDeviceData();

        //merge the data from the MNC into the resulting array
        for ($i = 0; $i < sizeof($result_devices); $i++) {
            for ($j = 0; $j < sizeof($MNCDevices['simpleDevices']); $j++) {
                if ($result_devices[$i]['mac'] == $MNCDevices['simpleDevices'][$j]['mac']) {
                    //merge the data from the MNC into the default data (like connectedMac and other technical details with aren't stored in this database)
                    $result_devices[$i] = $result_devices[$i] + $MNCDevices['simpleDevices'][$j];//plus operator will not override items on the left operand
                    break;
                }
            }
            //TODO remove this once it's fixed; correct invalid model#s for some devices
            $result_devices[$i]['modelName'] = self::fixMalformattedModelName($result_devices[$i]['modelName']);

            //check if any of the devices are new
            $deviceIsNew = self::findDeviceByMac($result_devices[$i]['mac'], $original_devices);
            if ($deviceIsNew < 0) {
                $result_devices[$i]['isNewDevice'] = true;
            }

        }

        //set the activeSource data (what source is currently connected to the RX)
        self::buildActiveSourceData($result_devices);

        return $result_devices;
    }





    public static function getBlockedUsersForDevice($mac) {
        if (empty($mac)) {
            return array(
                'status' => 'error',
                'message' => 'device cannot be empty'
            );
        }

        $data = array(
            'mac' => $mac
        );

        $sql = 'SELECT blocked_devices.*, users.email, users.role_type, users.username, users.enabled FROM blocked_devices LEFT JOIN users ON blocked_devices.user_id = users.user_id WHERE blocked_devices.mac = ?:[mac,s]';
        return DB::query($sql, $data);
    }

    public static function getBlockedUsersForDevices() {
        return DB::query('SELECT * FROM blocked_devices', array());
    }

    public static function blockUsers($mac, $users) {
        if (empty($mac)) {
            return array(
                'status' => 'error',
                'message' => 'device cannot be empty'
            );
        }

        if (empty($users)) {
            DB::query('DELETE FROM blocked_devices WHERE mac = ?:[mac,s]', array('mac' => $mac));
        } else {
            $data = array(
                'mac' => $mac
            );

            $sql = 'REPLACE INTO blocked_devices (mac, user_id) VALUES ';

            $values = array();
            for ($i = 0; $i < sizeof($users); $i++) {
                $values[] = ' (?:[mac,s], ' . intval($users[$i]) . ') ';
            }

            $sql .= implode(', ', $values);
            DB::query($sql, $data);

            return array(
                'status' => 'success'
            );
        }
    }


    /**
     * DIRTY FIX. the 762 reports its modelName not according to the standardized format
     * This has been reported, and until it gets fixed, we will have to adjust the format over here
     * TODO remove this when it's fixed
     *
     * * @param $modelName
     * @return string
     */
    public static function fixMalformattedModelName($modelName) {

        if (empty($modelName)) {
            return '';
        }

        $modelName = strtoupper($modelName);
        if (strpos($modelName, 'RX-') === 0) {
            $modelName = str_replace('RX-', '', $modelName) . '-RX';
        } else if (strpos($modelName, 'TX-') === 0) {
            $modelName = str_replace('TX-', '', $modelName) . '-TX';
        }
        return $modelName;
    }


    private static function processSimpleDevicePayloadFromMNC($devices) {

        if (empty($devices)) {
            $devices = array();
        }

        $all_simple_devices = self::getSimpleDevices();

        if (!empty($devices)) {
            foreach ($devices as $device) {
                //check if the device exists by mac property of array
                $exists = false;
                for ($i = 0; $i < sizeof($all_simple_devices); $i++) {
                    if (strtolower($device['mac']) == strtolower($all_simple_devices[$i]['mac'])) {
                        $exists = $i;
                    }
                }
                if ($exists === false) {
                    self::addDevice(array(
                        'customName' => $device['customName'],
                        'system_id' => $device['system_id'],
                        'mac' => $device['mac'],
                        'modelName' => self::fixMalformattedModelName($device['modelName']),
                    ));
                } else {

                    $fields = array();

                    //update the model number if necessary (this can happen where a user will flip the dipswitch on
                    if (strtolower($device['modelName']) != strtolower($all_simple_devices[$exists]['modelName'])) {
                        $fields['modelName'] = self::fixMalformattedModelName($device['modelName']);
                    }

                    if (strtolower($device['system_id']) != strtolower($all_simple_devices[$exists]['system_id'])) {
                        $fields['system_id'] = $device['system_id'];
                    }

                    if (strtolower($device['customName']) != strtolower($all_simple_devices[$exists]['customName'])) {
                        $fields['customName'] = $device['customName'];
                    }

                    self::updateDevice($device['mac'], $fields);
                }

            }
        }

        //check if any existing devices are no longer found in the MNC data
        foreach ($all_simple_devices as $angelicao_device) {
            $exists = false;
            $isDisconnected = false;
            foreach ($devices as $mnc_device) {
                if (strtolower($angelicao_device['mac']) == strtolower($mnc_device['mac'])) {
                    $exists = true;

                    if (!empty($angelicao_device['isDisconnected'])) {
                        $isDisconnected = true;
                    }
                }
            }


            if (empty($exists)) {
                $data = array(
                    'isDisconnected' => true,
                    'mac' => $angelicao_device['mac']
                );
                $sql = 'UPDATE devices SET isDisconnected = ?:[isDisconnected,b] WHERE mac LIKE ?:[mac,s]';
                DB::query($sql, $data);

            } else {//if the device is already in the data

                if ($isDisconnected === true) {//if the device was previously marked as disconnected, undo that.
                    $data = array(
                        'isDisconnected' => false,
                        'mac' => $angelicao_device['mac']
                    );
                    $sql = 'UPDATE devices SET isDisconnected = ?:[isDisconnected,b] WHERE mac LIKE ?:[mac,s]';
                    DB::query($sql, $data);

                }
            }

        }

    }

    
    public static function addDevice($device) {
        if (empty($device['customName'])) {
            return array(
                'status' => 'error',
                'message' => 'Name cannot be empty'
            );
        }
        if (!isset($device['system_id'])) {
            return array(
                'status' => 'error',
                'message' => 'system_id cannot be empty'
            );
        }
        if (empty($device['mac'])) {
            return array(
                'status' => 'error',
                'message' => 'mac cannot be empty'
            );
        }
        if (empty($device['modelName'])) {
            return array(
                'status' => 'error',
                'message' => 'Model cannot be empty'
            );
        }

        $data = array(
            'customName' => $device['customName'],
            'system_id' => $device['system_id'],
            'mac' => $device['mac'],
            'modelName' => $device['modelName'],
            'color' => ''
        );
        if (strpos(strtolower($data['modelName']), '-tx') !== false) {
            $data['color'] = SourceColorGenerator::generateColor();
        }

        //USE replace into instead of INSERT for cases like the 755 where the same product can be switched into RX or TX (and it's model name changes but not it's MAC)
        $sql = 'REPLACE INTO devices SET customName = ?:[customName,s], system_id = ?:[system_id,i], mac = ?:[mac,s], modelName = ?:[modelName,s], icon = "", location_id = NULL, deleted = 0, translate_x = NULL, translate_y = NULL, color = ?:[color,s]';
        DB::query($sql, $data);

        return $data;
    }

    public static function editIcon($mac, $icon) {
        if (empty($mac)) {
            return array(
                'status' => 'error',
                'message' => 'MAC cannot be empty'
            );
        }
        if (empty($icon)) {
            return array(
                'status' => 'error',
                'message' => 'Icon cannot be empty'
            );
        }

        $data = array(
            'mac' => $mac,
            'icon' => $icon
        );
        $sql = 'UPDATE devices SET icon = ?:[icon,s] WHERE mac = ?:[mac,s] LIMIT 1';
        DB::query($sql, $data);

        return array(
            'status' => 'success'
        );
    }

    public static function deleteDevice($mac) {
        if (empty($mac)) {
            return array(
                'status' => 'error',
                'message' => 'MAC cannot be empty'
            );
        }

        $data = array(
            'mac' => $mac
        );
        $sql = 'UPDATE devices SET deleted = 1 WHERE mac = ?:[mac,s] LIMIT 1';
        DB::query($sql, $data);

        return array(
            'status' => 'success'
        );
    }

    /**
     * Undelete is now a word.
     * @param $mac
     * @return array
     * @throws \Exception
     */
    public static function undeleteDevice($mac) {
        if (empty($mac)) {
            return array(
                'status' => 'error',
                'message' => 'MAC cannot be empty'
            );
        }

        $data = array(
            'mac' => $mac
        );
        $sql = 'UPDATE devices SET deleted = 0 WHERE mac = ?:[mac,s] LIMIT 1';
        DB::query($sql, $data);

        return array(
            'status' => 'success'
        );
    }

    public static function getDevice($mac) {
        if (empty($mac)) {
            return array(
                'status' => 'error',
                'message' => 'MAC cannot be empty'
            );
        }

        $user_account = MNC_API::getAccount();
        $data = array(
            'mac' => $mac,
            'user_id' => intval($user_account['user_id'])
        );
        $sql = 'SELECT devices.*, video_wall_layouts.name as `connectedLayout`, ' .
            '(SELECT COUNT(*) FROM video_wall_currently_playing_connections WHERE videoWallFullConfigId = devices.videoWallFullConfigId) as `connectedLayoutChanges` FROM devices ' .
            'LEFT JOIN video_wall_layouts ON (devices.selected_layout = video_wall_layouts.layout_id) WHERE devices.mac = ?:[mac,s] AND devices.mac NOT IN (select mac FROM blocked_devices WHERE user_id = ?:[user_id,i])';
        $result = DB::query($sql, $data);

        //var_dump($result);
        $device = array();
        if (!empty($result)) {
            $device = reset($result);

            if (self::isDeviceDisplay($device)) {
                $mnc_devices_by_mac = MNC_API::getDevicesByMac($device['system_id'], array($device['mac']));
                $mnc_device = array();

                if (!empty($mnc_devices_by_mac[$device['mac']])) {
                    $mnc_device = $mnc_devices_by_mac[$device['mac']];
                }

                if (!empty($mnc_device)) {
                    if (!empty($mnc_device['modelName'])) {
                        $mnc_device['modelName'] = self::fixMalformattedModelName($mnc_device['modelName']);
                    }
                    $device = $device + $mnc_device;
                }

                $locations = Locations::getAllLocations();

                $device_path = '';

                if (isset($device['location_id'])) {
                    $device_location = Locations::utilFindLocationByID($device['location_id'], $locations);
                    if ($device_location !== null) {
                        $device_path = $device_location['name'];

                        $location_path = Locations::utilGetLocationPath($device_location, $locations);
                        if (sizeof($location_path)) {
                            $device_path .= ' > ' . implode(' > ', array_reverse($location_path));
                        }
                    }

                }

                $device['path'] = $device_path;


                self::buildActiveSourceData($device);
            }

        }

        if (!empty($device)) {
            return $device;
        } else {
            return array(
                'status' => 'error',
                'message' => 'device not found'
            );
        }

    }

    public static function updateDevice($mac, $fields) {
        if (empty($mac)) {
            return array(
                'status' => 'error',
                'message' => 'MAC cannot be empty'
            );
        }
        if (empty($fields)) {
            return array(
                'status' => 'error',
                'message' => 'fields cannot be empty'
            );
        }

        $data = array(
            'mac' => $mac
        );
        $sql = 'UPDATE devices SET ';
        $where_clauses = array();

        if (isset($fields['modelName'])) {
            $where_clauses[] = ' modelName = ?:[modelName,s] ';
            $data['modelName'] = $fields['modelName'];
        }

        if (isset($fields['customName'])) {
            $where_clauses[] = ' customName = ?:[customName,s] ';
            $data['customName'] = $fields['customName'];
        }

        if (isset($fields['system_id'])) {
            $where_clauses[] = ' system_id = ?:[system_id,i] ';
            $data['system_id'] = $fields['system_id'];
        }

        $sql .= ' ' . implode(', ', $where_clauses) . ' WHERE mac = ?:[mac,s] LIMIT 1 ';
        DB::query($sql, $data);

        return array(
            'status' => 'success'
        );
    }


    public static function updateName($mac, $customName) {
        if (empty($mac)) {
            return array(
                'status' => 'error',
                'message' => 'MAC cannot be empty'
            );
        }
        if (empty($customName)) {
            return array(
                'status' => 'error',
                'message' => 'Name cannot be empty'
            );
        }

        $data = array(
            'mac' => $mac,
            'customName' => $customName
        );
        $sql = 'UPDATE devices SET customName = ?:[customName,s] WHERE mac = ?:[mac,s] LIMIT 1';
        DB::query($sql, $data);

        return array(
            'status' => 'success'
        );
    }

    public static function setIsUsedInViewMode($mac, $isUsed) {
        if (empty($mac)) {
            return array(
                'status' => 'error',
                'message' => 'MAC cannot be empty'
            );
        }

        $data = array(
            'mac' => $mac,
            'isUsedInViewMode' => (empty($isUsed) ? false : true)
        );
        $sql = 'UPDATE devices SET isUsedInViewMode = ?:[isUsedInViewMode,i] WHERE mac = ?:[mac,s] LIMIT 1';
        DB::query($sql, $data);

        return array(
            'status' => 'success'
        );
    }

    public static function getSourcesInSystem($system_id) {
        if (!is_numeric($system_id)) {
            return array(
                'status' => 'error',
                'message' => 'system id must be specified'
            );
        }

        $user_account = MNC_API::getAccount();
        $data = array(
            'system_id' => $system_id,
            'user_id' => intval($user_account['user_id'])
        );

        $sql = 'SELECT * FROM devices WHERE (LOCATE("-TX", UPPER(modelName)) > 0 OR LOCATE("TX-", UPPER(modelName)) > 0) AND system_id = ?:[system_id,i] AND deleted != 1 AND isDisconnected != 1 AND mac NOT IN (select mac from blocked_devices where user_id = ?:[user_id,i])';

        $results = DB::query($sql, $data);

        return $results;
    }

    public static function getDisplaysInSystem($system_id) {
        if (!is_numeric($system_id)) {
            return array(
                'status' => 'error',
                'message' => 'system id must be specified'
            );
        }

        $user_account = MNC_API::getAccount();

        $data = array(
            'system_id' => $system_id,
            'user_id' => intval($user_account['user_id'])
        );

        $sql = 'SELECT devices.*, video_wall_layouts.name as `connectedLayout`, ' .
            '(SELECT COUNT(*) FROM video_wall_currently_playing_connections WHERE videoWallFullConfigId = devices.videoWallFullConfigId) as `connectedLayoutChanges` FROM devices ' .
            'LEFT JOIN video_wall_layouts ON (devices.selected_layout = video_wall_layouts.layout_id) WHERE (LOCATE("-RX", UPPER(devices.modelName)) > 0 OR LOCATE("RX-", UPPER(devices.modelName)) > 0 OR devices.isVideoWall = 1) AND devices.system_id = ?:[system_id,i] AND devices.deleted != 1 AND devices.mac NOT IN (SELECT mac FROM video_wall_config_devices ) AND devices.mac NOT IN (select mac FROM blocked_devices WHERE user_id = ?:[user_id,i])';

        $results = DB::query($sql, $data);

        if (!empty($results)) {
            $macs = array();
            foreach ($results as $result) {
                $macs[] = $result['mac'];
            }

            $mnc_devices_by_mac = MNC_API::getDevicesByMac($system_id, $macs);

            for ($i = 0; $i < sizeof($results); $i++) {
                if ( !empty($mnc_devices_by_mac[$results[$i]['mac']]) ) {
                    if (!empty($mnc_devices_by_mac[$results[$i]['mac']]['modelName'])) {
                        $mnc_devices_by_mac[$results[$i]['mac']]['modelName'] = self::fixMalformattedModelName($mnc_devices_by_mac[$results[$i]['mac']]['modelName']);
                    }
                    $results[$i] = $results[$i] + $mnc_devices_by_mac[$results[$i]['mac']];
                }
            }

        }

        //get the MNC details for each device
        self::buildActiveSourceData($results);

        return $results;
    }


    /**
     * Removes a device from a its location - whatever the location is
     * @param $mac
     * @return array
     * @throws \Exception
     */
    public static function removeDeviceFromLocation($mac) {
        if (empty($mac)) {
            return array(
                'status' => 'error',
                'message' => 'you must specify a device'
            );
        }

        //delete the selected layout and set the current full VW source (bypass field)
        $data = array(
            'mac' => $mac
        );
        $sql = 'UPDATE devices SET location_id = NULL WHERE mac = ?:[mac,s] LIMIT 1';
        DB::query($sql, $data);

        return array(
            'status' => 'success'
        );
    }

    public static function getSourcesAvailableForDisplay($display_mac) {
        if (empty($display_mac)) {
            return array(
                'status' => 'error',
                'message' => 'display must not be empty'
            );
        }

        $user_account = MNC_API::getAccount();

        $sources = array();

        $data = array(
            'mac' => $display_mac,
            'user_id' => intval($user_account['user_id'])
        );

        $sql = 'SELECT * FROM devices WHERE mac = ?:[mac,s] AND mac NOT IN (SELECT mac FROM blocked_devices WHERE user_id = ?:[user_id,i])';
        $result_display = DB::query($sql, $data);
        $result_display = reset($result_display);

        //check if valid display
        if (!self::isDeviceDisplay($result_display)) {
            return array(
                'status' => 'error',
                'message' => 'device must be a display'
            );
        }

        //check if it's in a location
        if (!emptynz($result_display['location_id'])) {
            $preferred_locations = Locations::getPreferredSources($result_display['location_id'], $result_display['system_id']);
            for ($i = 0; $i < sizeof($preferred_locations); $i++) {
                $preferred_locations[$i]['isPreferred'] = true;
            }

            if (!empty($preferred_locations)) {
                $sources['preferred_sources'] = $preferred_locations;
            }

        }

        //otherwise return sources in system
        $sources_in_system = self::getSourcesInSystem($result_display['system_id']);
        $sources['sources_in_system'] = $sources_in_system;
        return $sources;
    }



    /**
     *
     *
     *
     *
     *
     *
     *
     *
     * util functions
     *
     *
     *
     *
     *
     *
     *
     */

    private static function findDeviceByMac($mac, $devices) {
        if (empty($devices)) {
            return -1;
        }

        $idx = -1;
        for ($i = 0; $i < sizeof($devices); $i++) {
            if (!empty($devices[$i]['mac']) && $devices[$i]['mac'] === $mac) {
                $idx = $i;
            }
        }

        return $idx;
    }

    private static function isDeviceDisplay($device) {
        $isDeviceDisplay = false;
        if (!empty($device['modelName'])) {
            $modelName = strtoupper($device['modelName']);
            if (strpos($modelName, '-RX') !== false) {
                $isDeviceDisplay = true;
            }

            if (!empty($device['isVideoWall'])) {
                $isDeviceDisplay = true;
            }

            if (!empty($device['mac']) && strpos(strtolower($device['mac']), '_vw_') !== false) {
                $isDeviceDisplay = true;
            }
        }

        return $isDeviceDisplay;
    }

    public static function buildActiveSourceData(&$data) {
        if (empty($data)) {
            return array();
        }

        //get all sources
        $sql =
            'SELECT devices.* FROM devices ' .
            ' WHERE LOCATE("-TX", UPPER(modelName)) > 0 ';
        $sources = DB::query($sql, array());

        if (!empty($data['mac']) && self::isDeviceDisplay($data)) {//ie this is an individual TV

            //set the activeSource data (what source is currently connected to the RX)
            if (!empty($data['connectedMac'])) {
                $connectedSource = self::findDeviceByMac($data['connectedMac'], $sources);
                if ($connectedSource >= 0) {
                    $data['activeSource'] = $sources[$connectedSource];
                }
            } else if (!empty($data['connectedLayout']) && empty($data['bypass_layout_source'])) {
                $data['activeSource'] = array(
                    'customName' => $data['connectedLayout']
                );
                if (!empty($data['connectedLayoutChanges']) && is_numeric($data['connectedLayoutChanges'])) {
                    $data['activeSource']['customName'] .= '*';
                }
            } else if (!empty($data['bypass_layout_source']) && empty($data['connectedLayout'])) {
                $connectedSource = self::findDeviceByMac($data['bypass_layout_source'], $sources);
                if ($connectedSource >= 0) {
                    $data['activeSource'] = $sources[$connectedSource];
                }

                if (!empty($data['connectedLayoutChanges']) && is_numeric($data['connectedLayoutChanges'])) {
                    $data['activeSource'] = array(
                        'customName' => 'Multiple Sources'
                    );
                }
            }

        } else {//ie we're dealing with many TVs
            for ($i = 0; $i < sizeof($data); $i++) {

                //set the activeSource data (what source is currently connected to the RX)
                if (!empty($data[$i]['connectedMac'])) {
                    $connectedSource = self::findDeviceByMac($data[$i]['connectedMac'], $sources);
                    if ($connectedSource >= 0) {
                        $data[$i]['activeSource'] = $sources[$connectedSource];
                    }
                } else if (!empty($data[$i]['connectedLayout']) && empty($data[$i]['bypass_layout_source'])) {
                    $data[$i]['activeSource'] = array(
                        'customName' => $data[$i]['connectedLayout']
                    );
                    if (!empty($data[$i]['connectedLayoutChanges']) && is_numeric($data[$i]['connectedLayoutChanges'])) {
                        $data[$i]['activeSource']['customName'] .= '*';
                    }
                } else if (!empty($data[$i]['bypass_layout_source']) && empty($data[$i]['connectedLayout'])) {
                    $connectedSource = self::findDeviceByMac($data[$i]['bypass_layout_source'], $sources);
                    if ($connectedSource >= 0) {
                        $data[$i]['activeSource'] = $sources[$connectedSource];
                    }

                    if (!empty($data[$i]['connectedLayoutChanges']) && is_numeric($data[$i]['connectedLayoutChanges'])) {
                        $data[$i]['activeSource'] = array(
                            'customName' => 'Multiple Sources'
                        );
                    }
                }

            }
        }

        //no need to return; data was passed by ref
    }
}